﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using System;
using UnityEngine.UI;

public class Fast_Movement_Pattern : MonoBehaviour
{
    public Camera VRCamera;
    private float EdgePointYFast;
    private float EdgePointXFast;
    public Texture Quick_Pattern;
    public Texture Still_Pattern;
    public Texture T_Default;
    public Renderer T_Renderer;
    private bool IsNotMoving;
    private bool FinishedUp;
    private bool FinishedDown;    
    private float StartFastTimer;
    private float Still = 0;

    bool IsLocked = false;
    GameObject CameraObj;
    PatternRecognition PatternScript;

    public GameObject LockedObj;
    public Vector3 PlaceOfHit;
    private GameObject SelectedObj;
    SelectionManager SelectionScript;
    [SerializeField]
    private TextMesh ObjectNameText;
    private bool Reseted = false;
    private const float limiter = 0.3f;


    // Use this for initialization
    void Start()
    {
        CameraObj = GameObject.Find("VRDisplayTracked");
        PatternScript = CameraObj.GetComponent<PatternRecognition>();

        SelectedObj = GameObject.Find("Selection Manager");
        SelectionScript = SelectedObj.GetComponent<SelectionManager>();
        ResetFastMovement();        
    }

    // Update is called once per frame
    void Update()
    {
        if (!PatternScript.IsLocked)
        {
            if (!IsNotMoving)
            { //checking if head is in still position            
                IsNotMoving = CheckStopMovement();
            }
            else if (!FinishedUp && !FinishedDown && IsNotMoving)
            { //checking of the movement up
                FinishedUp = MovementUp();
            }
            else if (FinishedUp && !FinishedDown)
            { // checking of the movement down
                FinishedDown = MovementDown();
            }
            else if (FinishedDown)
            {
                if (SelectionScript.hit.distance > 5.0f)
                {
                    MoveCamera();                 
                }
                ResetFastMovement();
            }
            
            float t = Time.time - StartFastTimer; //if each part of animation is not done under 2s (thus it being not intended), reset it
            if (t > limiter + 0.1f)
            {
                ObjectNameText.text = "";
                LockedObj = null;
                ResetFastMovement();
            }
        }
    }

    /* * function which returns if movement of cursor stopped
     * */
    bool CheckStopMovement()
    {       
        if (Math.Abs(EdgePointXFast - GetXrotation()) < 1.0f)
        { //if camera goes higher, change the last rotation in X coordinate
            if (Math.Abs(EdgePointYFast - GetYrotation()) < 1.0f)
            {
                if (Still == 0f)
                {
                    SetStill(Time.time);
                    EdgePointXFast = GetXrotation();
                    EdgePointYFast = GetYrotation();
                }
                else if (Time.time - Still > limiter)
                {
                    ResetFastTimer();
                    T_Renderer.material.mainTexture = Still_Pattern;
                    ObjectNameText.text = SelectionScript.ObjectNameText.text;
                    if (LockedObj != null)
                    {
                        LockedObj = SelectionScript.SelectedObject;
                    }
                    PlaceOfHit = SelectionScript.PlaceOfHit;
                    return true;
                }               
                return false;                
            }            
        }
        ResetFastMovement();        
        return false;
    }

    /* * function which returns if movement up was done correctly
     * */
    bool MovementUp()
    {
        if ((EdgePointYFast - GetYrotation() < 5.0f) && (EdgePointYFast - GetYrotation() > -5.0f))
        { //checks if the movement was in straight down field            
            if (EdgePointXFast - GetXrotation() > 3.0f)
            {
                EdgePointXFast = GetXrotation();
                EdgePointYFast = GetYrotation();                
                ResetFastTimer();
                return true;
            }
        }
        else
        {
            ResetFastMovement();
        }
        return false;
    }

    /* * function which returns if movement down was done correctly
     * */
    bool MovementDown()
    {
        if ((EdgePointYFast - GetYrotation() < 10.0f) && (EdgePointYFast - GetYrotation() > -10.0f))
        { //checks if the movement was in straight down field
            if (EdgePointXFast - GetXrotation() < -1.0f)
            {
                T_Renderer.material.mainTexture = Quick_Pattern;
                ResetFastTimer();
                new WaitForSeconds(0.5f);                
                return true;
            }
        }
        else
        {            
            ResetFastMovement();
        }
        return false;
    }

    /* * function which resets all the values needed for fast movement pattern animation
      * */
    public void ResetFastMovement()
    {        
        EdgePointXFast = GetXrotation();
        EdgePointYFast = GetYrotation();
        IsNotMoving = false;
        FinishedUp = false;
        FinishedDown = false;
        SetStill(0f);
        //m_Renderer.material.mainTexture = m_Default;
        ResetFastTimer();
    }

    /* * function which resets time needed for each part of animation
      * */
    public void ResetFastTimer()
    {
        StartFastTimer = Time.time;        
    }

    /* * function which sets up Still timer
     *  */
    void SetStill(float time)
    {
        Still = time;
    }

    /* * function which gives proper X (vertical) coordinates of where camera is looking (rotation)
      * */
    float GetXrotation()
    {
        float temp = VRCamera.transform.rotation.eulerAngles.x;
        return (temp > 270.0f) ? temp % 270 - 90.0f : temp;
    }

    /* * function which gives proper Y (horizontal) coordinates of where camera is looking (rotation)
      * */
    float GetYrotation()
    {
        return VRCamera.transform.rotation.eulerAngles.y;
    }

    /* Function which changes position of the camera */    
    void MoveCamera()
    {
        Vector3 Direction = VRCamera.transform.position + VRCamera.transform.rotation * Vector3.forward * 5f;
        Direction.y = 0.0f;
        transform.position = Direction;
    }
}