﻿using System;
using System.Collections;
using System.Collections.Generic;
using UnityEngine;

public class PatternRecognition : MonoBehaviour
{
    public Camera VRCamera;
    private float EdgePointY;
    private float EdgePointX;
    public Texture T_Default;
    public Texture T_Down;
    public Texture T_Right;
    public Texture T_Left;
    public Renderer T_Renderer;
    public GameObject Menu;
    public GameObject Cursor;
    public bool IsLocked = false;
    private bool FinishedDown;
    private bool FinishedRight;
    private bool FinishedLeft;
    private float StartTimer;
    int Counter = 0;
    public bool resetting = false;

    GameObject SelectedObj;
    Fast_Movement_Pattern FMPscript;

    GameObject ColorChangerObj;
    ColorChanger ColorChangerScript;

    GameObject SelectionManagerObj;
    SelectionManager SelectionManagerScript;

    // Use this for initialization
    void Start() {
        IsLocked = false;
        ResetMovement();
        CloseMenu();
        SelectedObj = GameObject.Find("VRDisplayTracked");
        FMPscript = SelectedObj.GetComponent<Fast_Movement_Pattern>();

        ColorChangerObj = GameObject.Find("Cube");
        ColorChangerScript = ColorChangerObj.GetComponent<ColorChanger>();

        SelectionManagerObj = GameObject.Find("Selection Manager");
        SelectionManagerScript = SelectionManagerObj.GetComponent<SelectionManager>();
    }

    // Update is called once per frame
    void Update() {  
        if (!FinishedDown && !FinishedRight && !FinishedLeft)
        { //checking of the movement down
            FinishedDown = MovementDown();                        
        }

        if (FinishedDown && !FinishedRight && !FinishedLeft)
        { //checking of the movement to the right            
            FinishedRight = MovementRight();
        }

        if (FinishedDown && FinishedRight && !FinishedLeft)
        { //checking of the movement to the left          
            FinishedLeft = MovementLeft();
        }

        if (FinishedLeft)
        { //if the animation is correctly finished, open menu
            if (Counter == 1)
            {                                
                OpenMenu();
            }
        }        
        float t = Time.time - StartTimer; //if each part of animation is not done under 2s (thus it being not intended), reset it
        if (t > 2.0f && Counter == 1)
       {
            IsLocked = false;
            ResetMovement();
        }
    }

    /* * function which returns if movement down was done correctly
     * */
    bool MovementDown()
    {        
        if ((EdgePointY - GetYrotation() < 10.0f) && (EdgePointY - GetYrotation() > -10.0f))
        { //checks if the movement was in straight down field            
            if (EdgePointX - GetXrotation() < -5.0f) {
                T_Renderer.material.mainTexture = T_Down;
                EdgePointX = GetXrotation();
                EdgePointY = GetYrotation();
                ResetTimer();
                IsLocked = true;
                return true;
            }
        }
        else
        {            
            ResetMovement();
        }
        return false;
    }

    /* * function which returns if movement to the up-right was done correctly
     * */
    bool MovementRight()
    {
        if (EdgePointY < 340.0f)
        {
            if ((((GetYrotation() - EdgePointY) - (-GetXrotation()) - EdgePointX) < 0.0f) || ((GetYrotation() - EdgePointY) - ((-GetXrotation()) - EdgePointX) > 20.0f))
            {
                ResetMovement();
                return false;
            }
        }

        if ((EdgePointX - GetXrotation() > 3.0f) && (EdgePointX - GetXrotation() < 15.0f))
        {
            if ((GetYrotation() - EdgePointY > 4.0f) || (GetYrotation() - EdgePointY < -300.0f))
            {
                T_Renderer.material.mainTexture = T_Right;
                EdgePointX = GetXrotation();
                EdgePointY = GetYrotation();
                ResetTimer();
                return true;
            }
        }
        return false;
    }

    /* * function which returns if movement to the left was done correctly
     * */
    bool MovementLeft() {
        if ((EdgePointX - GetXrotation() < 10.0f) && (EdgePointX - GetXrotation() > -10.0f))
        { // checks if movement was in straight left field
            if ((EdgePointY - GetYrotation() > 10.0f && EdgePointY - GetYrotation() < 50.0f) || (EdgePointY < 20.0f) && (EdgePointY - (GetYrotation()) < -330.0f)) 
            {
                T_Renderer.material.mainTexture = T_Left;
                ResetTimer();                
                return true;
            }
        }
        else
        {
            ResetMovement();
        }
        return false;
    }




    /* * function which resets all the values needed for animation
      * */
    public void ResetMovement() {                
        EdgePointX = GetXrotation();        
        EdgePointY = GetYrotation();
        FinishedDown = false;
        FinishedRight = false;
        FinishedLeft = false;
        T_Renderer.material.mainTexture = T_Default;
        IsLocked = false;
        Counter = 1;
        ResetTimer();        
    }

    /* * function which resets time needed for each part of animation
      * */
    void ResetTimer() {
        StartTimer = Time.time;
    }

    /* * function which gives proper X (vertical) coordinates of where camera is looking (rotation)
      * */
    float GetXrotation() {
        float temp = VRCamera.transform.rotation.eulerAngles.x;
        return (temp > 270.0f) ? temp % 270 - 90.0f : temp;
    }

    /* * function which gives proper Y (horizontal) coordinates of where camera is looking (rotation)
      * */
    float GetYrotation() {
        return VRCamera.transform.rotation.eulerAngles.y;
    }

    /* function which opens up the menu */
    public void OpenMenu()
    {
        Vector3 LookPointCam = Menu.gameObject.transform.position;        
        LookPointCam = VRCamera.transform.position + VRCamera.transform.rotation * Vector3.forward;
        Menu.gameObject.transform.position = LookPointCam;
        Menu.gameObject.transform.LookAt(VRCamera.transform.position);
        Menu.gameObject.transform.Rotate(0.0f, 180.0f, 0.0f);
        Counter = 0;
        Menu.gameObject.SetActive(true);        
    }

    /*function which closes menu and resets all the values needed for animation*/
    public void CloseMenu()
    {
        ResetMovement();
        Menu.gameObject.SetActive(false);                
    }
}
